@@ -11,16 +11,15 @@ def test_xplaneapi():
 
     # Modify and read value at same time
     for i in range(0, 100, 1):
-        client.setDataRef("sim/cockpit2/engine/actuators/throttle_ratio[0]", str(i/100))
+        if client.setDataRef("sim/cockpit2/engine/actuators/throttle_ratio[0]", str(i/100)) == False:
+            print("command did not execute correctly!")
+
         dataRefVal = client.getDataRef("sim/cockpit2/engine/actuators/throttle_ratio[0]")
         print(f"Value received for dref = {dataRefVal}")
         time.sleep(0.005)
 
-    ## Example for monitoring a dataref value
-    # while(True):
-    #     dataRefVal = client.getDataRef("sim/cockpit2/engine/actuators/throttle_ratio[0]")
-    #     print(f"Value received for dref = {dataRefVal}")
-    #     time.sleep(0.005)
+    # Disconnect the client cleanly
+    client.disconnect()
 
 
 test_xplaneapi()
\ No newline at end of file
